/* vim: set sw=4 ts=4 expandtab: */
import akka.actor.{ TypedActor, ActorSystem, TypedProps }

trait Client {
    def start() : Unit
    def pong() : Unit
}

class ClientImpl(server : Server) extends Client {
    def start() : Unit = server.ping(this)
    def pong() : Unit = {
        println(Thread.currentThread().getName() + ": Pong!")
        TypedActor.context.stop(TypedActor.context.self)
        TypedActor.context.system.shutdown
    }
}

trait Server {
    def ping(client : Client) : Unit
}

class ServerImpl extends Server {
    def ping(client : Client) : Unit = {
        client.pong()
        println(Thread.currentThread().getName() + ": Ping!")
        TypedActor.context.stop(TypedActor.context.self)
    }
}

object TypedPingPong extends App {
    val system = ActorSystem("PingPong")
    val server : Server = TypedActor(system).typedActorOf(TypedProps[ServerImpl]())
    val client = TypedActor(system).typedActorOf(TypedProps(classOf[Client], new ClientImpl(server)))
    client.start()  // get the ball rolling
}
